@@ -481,7 +481,7 @@ sub _check_dh_exec {
         tag 'dh-exec-private-helper', $path;
     }
 
-    my ($dhe_subst, $dhe_install) = (0, 0);
+    my ($dhe_subst, $dhe_install, $dhe_filter) = (0, 0, 0);
     my $fd = $path->open;
     while (<$fd>) {
         if (/\$\{([^\}]+)\}/) {
@@ -499,16 +499,47 @@ sub _check_dh_exec {
             }
         }
         $dhe_install = 1 if / => /;
+        $dhe_filter = 1 if /\[[^\]]+\]/;
+        $dhe_filter = 1 if /<[^>]+>/;
+
+        if (  /^usr\/lib\/\$\{([^\}]+)\}\/?$/
+            ||/^usr\/lib\/\$\{([^\}]+)\}\/?\s+\/usr\/lib\/\$\{([^\}]+)\}\/?$/
+            ||/^usr\/lib\/\$\{([^\}]+)\}[^\s]+$/) {
+            my $sv = $1;
+            my $dv = $2;
+            my $dhe_useless = 0;
+
+            if (
+                $sv =~ m{ \A
+                   DEB_(?:BUILD|HOST)_(?:
+                       ARCH (?: _OS|_CPU|_BITS|_ENDIAN )?
+                      |GNU_ (?:CPU|SYSTEM|TYPE)|MULTIARCH
+             ) \Z}xsm
+              ) {
+                if (defined($dv)) {
+                    $dhe_useless = ($sv eq $dv);
+                } else {
+                    $dhe_useless = 1;
+                }
+            }
+            if ($dhe_useless) {
+                my $form = $_;
+                chomp($form);
+                $form = "\"$form\"";
+                tag 'dh-exec-useless-usage', $path, $form;
+            }
+        }
     }
     close($fd);
 
-    if (!($dhe_subst || $dhe_install)) {
+    if (!($dhe_subst || $dhe_install || $dhe_filter)) {
         tag 'dh-exec-script-without-dh-exec-features', $path;
     }
 
     if ($dhe_install && ($base ne 'install' && $base ne 'manpages')) {
         tag 'dh-exec-install-not-allowed-here', $path;
     }
+
     return;
 }
 
