@@ -37,8 +37,7 @@ my $TRIPLETS = Lintian::Data->new('files/triplets', qr/\s++/);
 my $LOCALE_CODES = Lintian::Data->new('files/locale-codes', qr/\s++/);
 my $INCORRECT_LOCALE_CODES
   = Lintian::Data->new('files/incorrect-locale-codes', qr/\s++/);
-my $MULTIARCH_DIRS = Lintian::Data->new('common/multiarch-dirs', qr/\s++/,
-    sub { return { 'dir' => $_[1], 'match' => qr/\Q$_[1]\E/ } });
+my $MULTIARCH_DIRS = Lintian::Data->new('common/multiarch-dirs', qr/\s++/);
 
 my $PRIVACY_BREAKER_WEBSITES= Lintian::Data->new(
     'files/privacy-breaker-websites',
@@ -447,7 +446,7 @@ sub run {
             or $fname =~ m,^var/lib/buildd/,
             or $fname =~ m,^build/,
             or $fname =~ m,^tmp/buildd/,) {
-            unless ($source_pkg eq 'sbuild') {
+            unless ($source_pkg eq 'sbuild' || $source_pkg eq 'pbuilder') {
                 tag 'dir-or-file-in-build-tree', $file;
             }
         }
@@ -657,17 +656,15 @@ sub run {
                     # check if pkgconfig file include path point to
                     # arch specific dir
                   MULTI_ARCH_DIR:
-                    foreach my $multiarch_dir ($MULTIARCH_DIRS->all) {
-                        my $value =  $MULTIARCH_DIRS->value($multiarch_dir);
-                        my $pkgconfig_dir = $value->{'dir'};
-                        my $regex = $value->{'match'};
-                        if ($pkg_config_arch eq $pkgconfig_dir) {
+                    foreach my $arch ($MULTIARCH_DIRS->all) {
+                        my $madir = $MULTIARCH_DIRS->value($arch);
+                        if ($pkg_config_arch eq $madir) {
                             next MULTI_ARCH_DIR;
                         }
-                        if ($block =~ m{\W$regex(\W|$)}xms) {
+                        if ($block =~ m{\W\Q$madir\E(\W|$)}xms) {
                             tag 'pkg-config-multi-arch-wrong-dir',$file,
                               'full text contains architecture specific dir',
-                              $pkgconfig_dir;
+                              $madir;
                             last MULTI_ARCH_DIR;
                         }
                     }
@@ -1014,7 +1011,7 @@ sub run {
                 # non-FHS, but still usual
                 or $fname =~ m,^usr/[^/]+-linux[^/]*/,
                 or $fname =~ m,^usr/iraf/,
-                # not allowed, but tested indivudually
+                # not allowed, but tested individually
                 or $fname =~ m{\A (?:
                         build|home|mnt|opt|root|run|srv
                        |(?:(?:usr|var)/)?tmp)|var/www/}xsm
