@@ -210,7 +210,7 @@ my @bashism_regexs = (
     $LEADIN . qr'type\s',
     $LEADIN . qr'time\s',
     $LEADIN . qr'dirs(\s|\Z)',
-    qr'(?:^|\s+)[<>]\(.*?\)',      # <() process substituion
+    qr'(?:^|\s+)[<>]\(.*?\)',      # <() process substitution
     qr'(?:^|\s+)readonly\s+-[af]', # readonly -[af]
     $LEADIN . qr'(sh|\$\{?SHELL\}?) -[rD]', # sh -[rD]
     $LEADIN . qr'(sh|\$\{?SHELL\}?) --\w+', # sh --long-option
@@ -594,7 +594,7 @@ sub run {
         if ($shellscript) {
             $checkbashisms = $base eq 'sh' ? 1 : 0;
             if ($base eq 'sh' or $base eq 'bash') {
-                if (check_script_syntax('/bin/bash', $path)) {
+                if (check_script_syntax("/bin/${base}", $path)) {
                     tag 'maintainer-shell-script-fails-syntax-check', $file;
                 }
             }
@@ -1200,7 +1200,7 @@ sub script_is_evil_and_wrong {
     return $ret;
 }
 
-# Given an interpretor and a file, run the interpretor on that file with the
+# Given an interpreter and a file, run the interpreter on that file with the
 # -n option to check syntax, discarding output and returning the exit status.
 sub check_script_syntax {
     my ($interpreter, $path) = @_;
