@@ -32,21 +32,6 @@ use Lintian::Path;
 use Lintian::Path::FSInfo;
 use Lintian::Util qw(fail open_gz perm2oct normalize_pkg_path dequote_name);
 
-my %INDEX_FAUX_DIR_TEMPLATE = (
-    'name'     => '',
-    'type'     => 'd',
-    'basename' => '',
-    'owner'    => 'root',
-    'group'    => 'root',
-    'size'     => 0,
-    'operm'    => 0755,
-    # Pick a "random" (but fixed) date
-    # - hint, it's a good read.  :)
-    'date'     => '1998-01-25',
-    'time'     => '22:55:34',
-    'faux'     => 1,
-);
-
 # A cache for (probably) the 5 most common permission strings seen in
 # the wild.
 # It may seem obscene, but it has an extreme "hit-ratio" and it is
@@ -59,6 +44,25 @@ my %PERM_CACHE = map { $_ => perm2oct($_) } (
     'lrwxrwxrwx', # symlinks
 );
 
+my %FILE_CODE2LPATH_TYPE = (
+    '-' => Lintian::Path::TYPE_FILE     | Lintian::Path::OPEN_IS_OK,
+    'h' => Lintian::Path::TYPE_HARDLINK | Lintian::Path::OPEN_IS_OK,
+    'd' => Lintian::Path::TYPE_DIR      | Lintian::Path::FS_PATH_IS_OK,
+    'l' => Lintian::Path::TYPE_SYMLINK,
+    'b' => Lintian::Path::TYPE_BLOCK_DEV,
+    'c' => Lintian::Path::TYPE_CHAR_DEV,
+    'p' => Lintian::Path::TYPE_PIPE,
+);
+
+my %INDEX_FAUX_DIR_TEMPLATE = (
+    'name'       => '',
+    '_path_info' => $FILE_CODE2LPATH_TYPE{'d'} | 0755,
+    # Pick a "random" (but fixed) date
+    # - hint, it's a good read.  :)
+    'date_time'  => '1998-01-25 22:55:34',
+    'faux'       => 1,
+);
+
 =head1 NAME
 
 Lintian::Collect::Package - Lintian base interface to binary and source package data collection
@@ -429,20 +433,30 @@ sub _fetch_index_data {
     while (my $line = <$idx>) {
         chomp($line);
 
-        my (%file, $perm, $owner, $name);
-        ($perm,$owner,$file{size},$file{date},$file{time},$name)
-          =split(' ', $line, 6);
+        my (%file, $perm, $operm, $ownership, $name, $raw_type, $size);
+        my ($date, $time);
+        ($perm,$ownership,$size,$date,$time,$name)=split(' ', $line, 6);
+
+        $file{'date_time'} = "${date} ${time}";
+        $raw_type = substr($perm, 0, 1);
+
+        # Only set size if it is non-zero and even then, only for
+        # regular files.  When we set it, insist on it being an int.
+        # This makes perl store it slightly more efficient.
+        $file{'size'} = int($size) if $size and $raw_type eq '-';
+
         # This may appear to be obscene, but the call overhead of
         # perm2oct is measurable on (e.g.) chromium-browser.  With
         # the cache we go from ~1.5s to ~0.1s.
         #   Of the 115363 paths here, only 306 had an "uncached"
         # permission string (chromium-browser/32.0.1700.123-2).
         if (exists($PERM_CACHE{$perm})) {
-            $file{operm} = $PERM_CACHE{$perm};
+            $operm = $PERM_CACHE{$perm};
         } else {
-            $file{operm} = perm2oct($perm);
+            $operm = perm2oct($perm);
         }
-        $file{type} = substr $perm, 0, 1;
+        $file{'_path_info'} = $operm
+          | ($FILE_CODE2LPATH_TYPE{$raw_type} // Lintian::Path::TYPE_OTHER);
 
         if ($num_idx) {
             # If we have a "numeric owner" index file, read that as well
@@ -452,21 +466,35 @@ sub _fetch_index_data {
             my ($owner_id, $name_chk) = (split(' ', $numeric, 6))[1, 5];
             croak "mismatching contents of index files: $name $name_chk"
               if $name ne $name_chk;
-            ($file{uid}, $file{gid}) = split '/', $owner_id, 2;
+            my ($uid, $gid) = split('/', $owner_id, 2);
+            # Memory-optimise for 0/0.  Perl has an insane overhead
+            # for each field, so this is sadly worth it!
+            if ($uid) {
+                $file{'uid'} = $uid;
+            }
+            if ($gid) {
+                $file{'gid'} = $gid;
+            }
         }
 
-        ($file{owner}, $file{group}) = split '/', $owner, 2;
+        my ($owner, $group) = split('/', $ownership, 2);
 
-        $file{owner} = 'root' if $file{owner} eq '0';
-        $file{group} = 'root' if $file{group} eq '0';
+        # Memory-optimise for root/root.  Perl has an insane overhead
+        # for each field, so this is sadly worth it!
+        if ($owner ne 'root' and $owner ne '0') {
+            $file{'owner'} = $owner;
+        }
+        if ($group ne 'root' and $group ne '0') {
+            $file{'group'} = $group;
+        }
 
         if ($name =~ s/ link to (.*)//) {
             my $target = dequote_name($1);
-            $file{type} = 'h';
+            $file{'_path_info'} = $FILE_CODE2LPATH_TYPE{'h'} | $operm;
             $file{link} = $target;
 
             push @{$rhlinks{$target}}, dequote_name($name);
-        } elsif ($file{type} eq 'l') {
+        } elsif ($raw_type eq 'l') {
             ($name, $file{link}) = split ' -> ', $name, 2;
             $file{link} = dequote_name($file{link}, 0);
         }
@@ -474,19 +502,13 @@ sub _fetch_index_data {
         # reason for storing it now is that we may need it during the
         # "hard-link fixup"-phase.
         $file{'name'} = $name = dequote_name($name);
-        $file{'_fs_info'} = $fs_info;
 
         $idxh{$name} = \%file;
 
         # Record children
-        $children{$name} ||= [] if $file{type} eq 'd';
-        my ($parent, $base) = ($name =~ m,^(.+/)?([^/]+/?)$,);
+        $children{$name} ||= [] if $raw_type eq 'd';
+        my ($parent) = ($name =~ m,^(.+/)?(?:[^/]+/?)$,);
         $parent = '' unless defined $parent;
-        $base = '' unless defined $base;
-        $file{basename} = $base;
-        # Insert the dirname field later for all (non-root) entries as
-        # it allows us to better reuse memory.
-        $file{dirname} = '' if $base eq '';
 
         $children{$parent} = [] unless exists $children{$parent};
 
@@ -505,18 +527,9 @@ sub _fetch_index_data {
         # check_dirs /can/ contain the same item multiple times.
         if (!exists($idxh{$name})) {
             my %cpy = %INDEX_FAUX_DIR_TEMPLATE;
-            my ($parent, $base) = ($name =~ m,^(.+/)?([^/]+/?)$,);
+            my ($parent) = ($name =~ m,^(.+/)?(?:[^/]+/?)$,);
             $parent //= '';
-            $base //= '';
-            $cpy{'_fs_info'} = $fs_info;
             $cpy{'name'} = $name;
-            $cpy{'basename'} = $base;
-            # Re: above, only insert dirname now for the root entry.
-            $cpy{'dirname'} = q{} if $name eq q{};
-            if ($num_idx) {
-                $cpy{'uid'} = 0;
-                $cpy{'gid'} = 0;
-            }
             $idxh{$name} = \%cpy;
             $children{$parent} = [] unless exists $children{$parent};
             push @{ $children{$parent} }, $name unless $parent eq $name;
@@ -537,7 +550,7 @@ sub _fetch_index_data {
             while (my $current = pop @check) {
                 $candidates{$current} = 1;
                 foreach my $rdep (@{$rhlinks{$current}}) {
-                    # There should not be any cicles, but just in case
+                    # There should not be any cycles, but just in case
                     push @check, $rdep unless $candidates{$rdep};
                 }
                 # Remove links we are fixing
@@ -556,14 +569,20 @@ sub _fetch_index_data {
                 next unless exists $idxh{$target};
                 my $le = $idxh{$link};
                 # We may be "demoting" a "real file" to a "hardlink"
-                $le->{type} = 'h';
+                $le->{'_path_info'}
+                  = ($le->{'_path_info'} & ~Lintian::Path::TYPE_FILE)
+                  | Lintian::Path::TYPE_HARDLINK;
                 $le->{link} = $target;
             }
             if ($target ne $e->{name}) {
-                $idxh{$target}->{type} = '-';
+                $idxh{$target}{'_path_info'}
+                  = ($idxh{$target}{'_path_info'}
+                      & ~Lintian::Path::TYPE_HARDLINK)
+                  | Lintian::Path::TYPE_FILE;
                 # hardlinks does not have size, so copy that from the original
                 # entry.
-                $idxh{$target}->{size} = $e->{size};
+                $idxh{$target}{'size'} = $e->{'size'} if exists($e->{'size'});
+                delete($e->{'size'});
                 delete $idxh{$target}->{link};
             }
         }
@@ -573,23 +592,20 @@ sub _fetch_index_data {
         # Add them in reverse order - entries in a dir are made
         # objects before the dir itself.
         my $entry = $idxh{$file};
-        if ($entry->{'type'} eq 'd') {
+        if ($entry->{'_path_info'} & Lintian::Path::TYPE_DIR) {
             my (%child_table, @sorted_children);
             for my $cname (sort(@{ $children{$file} })) {
                 my $child = $idxh{$cname};
                 my $basename = $child->basename;
-                # Insert dirname here to share the same storage with
-                # the hash key
-                $child->{'dirname'} = $file;
                 if (substr($basename, -1, 1) eq '/') {
                     $basename = substr($basename, 0, -1);
                 }
                 $child_table{$basename} = $child;
                 push(@sorted_children, $child);
             }
-            @sorted_children = reverse(@sorted_children);
             $entry->{'_sorted_children'} = \@sorted_children;
             $entry->{'children'} = \%child_table;
+            $entry->{'_fs_info'} = $fs_info;
         }
         # Insert name here to share the same storage with the hash key
         $entry->{'name'} = $file;
