@@ -543,7 +543,7 @@ sub file_overrides {
                   (?: \s*+ \[([^\]]+?)\])?          # optionally followed by an [arch-list] (like in B-D) -> $2
                   (?:\s*+ ([a-z]+) \s*+ )?          # optionally followed by the type -> $3
                 :\s++)?                             # end optional part
-                ([\-\.a-zA-Z_0-9]+ (?:\s.+)?)       # <tag-name> [extra] -> $4
+                ([\-\+\.a-zA-Z_0-9]+ (?:\s.+)?)     # <tag-name> [extra] -> $4
                    \Z/xsm
           ) {
             # Valid - so far at least
@@ -644,7 +644,7 @@ sub file_overrides {
                 # So, we got an override that does not start with the
                 # package name - cases include:
                 #  1 <tag> ...
-                #  2 <tag> someting: ...
+                #  2 <tag> something: ...
                 #  3 <wrong-pkg> [archlist] <type>: <tag> ...
                 #  4 <wrong-pkg>: <tag> ...
                 #  5 <wrong-pkg> <type>: <tag> ...
@@ -673,7 +673,7 @@ sub file_overrides {
                 }
             }
             # Nope, package name appears to match (or not present
-            # atall), not sure what the problem is so we just throw a
+            # at all), not sure what the problem is so we just throw a
             # generic parse error.
 
             tag 'malformed-override', "Cannot parse line $.: $_";
@@ -777,7 +777,7 @@ sub overrides {
 
 =item statistics([PROC])
 
-Returns a reference to the statistics hash for the given proccessable or, if PROC
+Returns a reference to the statistics hash for the given processable or, if PROC
 is omitted, a reference to the full statistics hash for all files.  In the
 latter case, the returned hash reference has as keys the file names and as
 values the per-file statistics.
