@@ -54,6 +54,7 @@ BEGIN {
           visit_dpkg_paragraph
           parse_dpkg_control
           read_dpkg_control
+          read_dpkg_control_utf8
           dpkg_deb_has_ctrl_tarfile
           get_deb_info
           get_dsc_info
@@ -120,7 +121,7 @@ Lintian::Util - Lintian utility functions
  }
  
  my (@paragraphs);
- eval { @paragraphs = read_dpkg_control('some/debian/ctrl/file'); };
+ eval { @paragraphs = read_dpkg_control_utf8('some/debian/ctrl/file'); };
  if ($@) {
     # syntax error etc.
     die "ctrl/file: $@";
@@ -164,14 +165,14 @@ You have a I<.dsc> (or I<.changes>) file.  Alternative, it is also
 useful if you have a control file and only care about the first
 paragraph.
 
-=item Use L</read_dpkg_control> when
+=item Use L</read_dpkg_control_utf8> or L</read_dpkg_control> when
 
 You have a debian control file (such I<debian/control>) and you want
 a number of paragraphs from it.
 
 =item Use L</parse_dpkg_control> when
 
-When you would have used L</read_dpkg_control>, except you have an
+When you would have used L</read_dpkg_control_utf8>, except you have an
 open filehandle rather than a file name.
 
 =back
@@ -594,6 +595,8 @@ sub visit_dpkg_paragraph {
     }
 }
 
+=item read_dpkg_control_utf8(FILE[, FLAGS[, LINES]])
+
 =item read_dpkg_control(FILE[, FLAGS[, LINES]])
 
 This is a convenience function to ease using L</parse_dpkg_control>
@@ -605,7 +608,7 @@ Otherwise, this behaves like:
 
  use autodie;
  
- open(my $fd, '<', FILE);
+ open(my $fd, '<:encoding(UTF-8)', FILE); # or '<'
  my @p = parse_dpkg_control($fd, FLAGS, LINES);
  close($fd);
  return @p;
@@ -626,6 +629,16 @@ sub read_dpkg_control {
     return @data;
 }
 
+sub read_dpkg_control_utf8 {
+    my ($file, $flags, $lines) = @_;
+
+    open(my $CONTROL, '<:encoding(UTF-8)', $file);
+    my @data = parse_dpkg_control($CONTROL, $flags, $lines);
+    close($CONTROL);
+
+    return @data;
+}
+
 =item dpkg_deb_has_ctrl_tarfile()
 
 Check if lintian could use dpkg-deb instead of ar and tar
@@ -875,6 +888,7 @@ The list of whitelisted %ENV variables are:
 
  PATH
  LC_ALL (*)
+ TMPDIR
 
 (*) LC_ALL is a special case as clean_env will change its value to
 either "C.UTF-8" or "C" (if CLOC is given and a truth value).
@@ -883,7 +897,7 @@ either "C.UTF-8" or "C" (if CLOC is given and a truth value).
 
 sub clean_env {
     my ($cloc) = @_;
-    my @whitelist = qw(PATH);
+    my @whitelist = qw(PATH TMPDIR);
     my %newenv
       = map { exists $ENV{$_} ? ($_ => $ENV{$_}) : () } (@whitelist);
     %ENV = %newenv;
@@ -1129,7 +1143,7 @@ Otherwise, a copy of the string is returned:
   while ( my $orig = <$fd> ) {
     my $stripped = strip ($orig);
     if ($stripped ne $orig) {
-        # $orig had leadning or/and trailing whitespace
+        # $orig had leading or/and trailing whitespace
     }
   }
 
